
/* ------------------------------------------------------------------------- */

/*

memory2.h

United States Naval Observatory
USNO/NRL Optical Interferometer
3450 Massachusetts Avenue, NW
Washington, DC  20392-5420

Subsystem:
----------
My software libraries

Description:
------------
This is the memory allocation/deallocation header file.

Modification history:
---------------------
1992 Jul 27 - Nicholas Elias, USNO/AD5
	      File created.
1997 May 17 - Nicholas Elias, USNO/AD5
              File memory.h renamed to memory2.h.

*/

/* ------------------------------------------------------------------------- */

/* Includes */

#include <malloc.h> /* Memory allocation */

/* ------------------------------------------------------------------------- */

/* Define the macros for the allocation and deallocation of memory and declare 
   their associated static global variables */

#define ALLOC1( pVar, TYPE, iNumRow ) \
    { \
      int iNumRowALLOC; \
      iNumRowALLOC = iNumRow + 1; \
      pVar = (TYPE*) \
          malloc( (unsigned int) ( iNumRowALLOC * sizeof( TYPE ) ) ); \
      if ( pVar == NULL ) \
      { \
        fprintf( stderr, "ALLOC1: Memory problem... \n" ); \
      } \
    } \
    /* Allocate memory for a 1D TYPE matrix between 0 and iNumRow */

#define ALLOC2( aVar, TYPE, iNumRow, iNumCol ) \
    { \
      int iCALLOC; \
      int iNumColALLOC; \
      int iNumMultALLOC; \
      int iNumRowALLOC; \
      iNumColALLOC = iNumCol + 1; \
      iNumRowALLOC = iNumRow + 1; \
      iNumMultALLOC = iNumRowALLOC * iNumColALLOC; \
      aVar = (TYPE**) \
          malloc( (unsigned int) ( iNumRowALLOC + 1 ) * sizeof( TYPE* ) ); \
      if ( aVar == NULL ) \
      { \
        fprintf( stderr, "ALLOC2: Memory problem...\n" ); \
      } \
      aVar += 1; \
      aVar[0] = (TYPE*) \
          malloc( (unsigned int) ( iNumMultALLOC + 1 ) * sizeof( TYPE ) ); \
      if ( aVar[0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC2: Memory problem...\n" ); \
      } \
      aVar[0] += 1; \
      for ( iCALLOC = 1; iCALLOC <= iNumRow; iCALLOC++ ) \
      { \
        aVar[iCALLOC] = aVar[iCALLOC - 1] + iNumColALLOC; \
      } \
    } /* Allocate the memory for a 2D TYPE matrix between (0,0) and \
         (iNumRow,iNumCol) */

#define ALLOC3( aVar, TYPE, iNumRow, iNumCol, iNumDep ) \
    { \
      int iCALLOC1; \
      int iCALLOC2; \
      int iNumColALLOC; \
      int iNumDepALLOC; \
      int iNumMultALLOC1; \
      int iNumMultALLOC2; \
      int iNumMultALLOC3; \
      int iNumRowALLOC; \
      iNumRowALLOC = iNumRow + 1; \
      iNumColALLOC = iNumCol + 1; \
      iNumDepALLOC = iNumDep + 1; \
      iNumMultALLOC1 = iNumRowALLOC * iNumColALLOC; \
      iNumMultALLOC2 = iNumRowALLOC * iNumColALLOC * iNumDepALLOC; \
      iNumMultALLOC3 = iNumColALLOC * iNumDepALLOC; \
      aVar = (TYPE***) \
          malloc( (unsigned int) ( iNumRowALLOC + 1 ) * sizeof( TYPE** ) ); \
      if ( aVar == NULL ) \
      { \
        fprintf( stderr, "ALLOC3: Memory problem...\n" ); \
      } \
      aVar += 1; \
      aVar[0] = (TYPE**) \
          malloc( (unsigned int) ( iNumMultALLOC1 + 1 ) * sizeof( TYPE* ) ); \
      if ( aVar[0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC3: Memory problem...\n" ); \
      } \
      aVar[0] += 1; \
      aVar[0][0] = (TYPE*) \
          malloc( (unsigned int) ( iNumMultALLOC2 + 1 ) * sizeof( TYPE ) ); \
      if ( aVar[0][0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC3: Memory problem...\n" ); \
      } \
      aVar[0][0] += 1; \
      for ( iCALLOC1 = 1; iCALLOC1 <= iNumCol; iCALLOC1++ ) \
      { \
        aVar[0][iCALLOC1] = aVar[0][iCALLOC1 - 1] + iNumDepALLOC; \
      } \
      for ( iCALLOC1 = 1; iCALLOC1 <= iNumRow; iCALLOC1++ ) \
      { \
        aVar[iCALLOC1] = aVar[iCALLOC1 - 1] + iNumColALLOC; \
        aVar[iCALLOC1][0] = aVar[iCALLOC1 - 1][0] + iNumMultALLOC3; \
        for ( iCALLOC2 = 1; iCALLOC2 <= iNumCol; iCALLOC2++ ) \
        { \
          aVar[iCALLOC1][iCALLOC2] = \
              aVar[iCALLOC1][iCALLOC2 - 1] + iNumDepALLOC; \
        } \
      } \
    } /* Allocate the memory for a 3D TYPE matrix between (0,0,0) and \
         (iNumRow,iNumCol,iNumDep) */ 

#define ALLOC4( aVar, TYPE, iNumRow, iNumCol, iNumDep1, iNumDep2 ) \
    { \
      int iCALLOC1; \
      int iCALLOC2; \
      int iCALLOC3; \
      int iNumColALLOC; \
      int iNumDep1ALLOC; \
      int iNumDep2ALLOC; \
      int iNumMultALLOC1; \
      int iNumMultALLOC2; \
      int iNumMultALLOC3; \
      int iNumMultALLOC4; \
      int iNumMultALLOC5; \
      int iNumMultALLOC6; \
      int iNumRowALLOC; \
      iNumRowALLOC = iNumRow + 1; \
      iNumColALLOC = iNumCol + 1; \
      iNumDep1ALLOC = iNumDep1 + 1; \
      iNumDep2ALLOC = iNumDep2 + 1; \
      iNumMultALLOC1 = iNumRowALLOC * iNumColALLOC; \
      iNumMultALLOC2 = iNumRowALLOC * iNumColALLOC * iNumDep1ALLOC; \
      iNumMultALLOC3 = iNumRowALLOC * iNumColALLOC * iNumDep1ALLOC \
          * iNumDep2ALLOC; \
      iNumMultALLOC4 = iNumColALLOC * iNumDep1ALLOC; \
      iNumMultALLOC5 = iNumColALLOC * iNumDep1ALLOC * iNumDep2ALLOC; \
      iNumMultALLOC6 = iNumDep1ALLOC * iNumDep2ALLOC; \
      aVar = (TYPE****) \
          malloc( (unsigned int) ( iNumRowALLOC + 1 ) * sizeof( TYPE*** ) ); \
      if ( aVar == NULL ) \
      { \
        fprintf( stderr, "ALLOC4: Memory problem...\n" ); \
      } \
      aVar += 1; \
      aVar[0] = (TYPE***) \
          malloc( (unsigned int) ( iNumMultALLOC1 + 1 ) * sizeof( TYPE** ) ); \
      if ( aVar[0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC4: Memory problem...\n" ); \
      } \
      aVar[0] += 1; \
      aVar[0][0] = (TYPE**) \
          malloc( (unsigned int) ( iNumMultALLOC2 + 1 ) * sizeof( TYPE* ) ); \
      if ( aVar[0][0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC4: Memory problem...\n" ); \
      } \
      aVar[0][0] += 1; \
      aVar[0][0][0] = (TYPE*) \
          malloc( (unsigned int) ( iNumMultALLOC3 + 1 ) * sizeof( TYPE ) ); \
      if ( aVar[0][0][0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC4: Memory problem...\n" ); \
      } \
      aVar[0][0][0] += 1; \
      for ( iCALLOC1 = 0; iCALLOC1 <= iNumRow; iCALLOC1++ ) \
      { \
        if ( iCALLOC1 != 0 ) \
        { \
          aVar[iCALLOC1] = aVar[iCALLOC1 - 1] + iNumColALLOC; \
          aVar[iCALLOC1][0] = aVar[iCALLOC1 - 1][0] + iNumMultALLOC4; \
          aVar[iCALLOC1][0][0] = aVar[iCALLOC1 - 1][0][0] + iNumMultALLOC5; \
        } \
        for ( iCALLOC2 = 0; iCALLOC2 <= iNumCol; iCALLOC2++ ) \
        { \
          if ( iCALLOC2 != 0 ) \
          { \
            aVar[iCALLOC1][iCALLOC2] = aVar[iCALLOC1][iCALLOC2 - 1] \
                + iNumDep1ALLOC; \
            aVar[iCALLOC1][iCALLOC2][0] = aVar[iCALLOC1][iCALLOC2 - 1][0] \
                + iNumMultALLOC6; \
          } \
          for ( iCALLOC3 = 0; iCALLOC3 <= iNumDep1; iCALLOC3++ ) \
          { \
            if ( iCALLOC3 != 0 ) \
            { \
              aVar[iCALLOC1][iCALLOC2][iCALLOC3] = \
                  aVar[iCALLOC1][iCALLOC2][iCALLOC3 - 1] + iNumDep2ALLOC; \
            } \
          } \
        } \
      } \
    } /* Allocate the memory for a 4D TYPE matrix between (0,0,0,0) and \
         (iNumRow,iNumCol,iNumDep1,iNumDep2) */

#define ALLOC5( aVar, TYPE, iNumRow, iNumCol, iNumDep1, iNumDep2, iNumDep3 ) \
    { \
      int iCALLOC1; \
      int iCALLOC2; \
      int iCALLOC3; \
      int iCALLOC4; \
      int iNumColALLOC; \
      int iNumDep1ALLOC; \
      int iNumDep2ALLOC; \
      int iNumDep3ALLOC; \
      int iNumMultALLOC1; \
      int iNumMultALLOC2; \
      int iNumMultALLOC3; \
      int iNumMultALLOC4; \
      int iNumMultALLOC5; \
      int iNumMultALLOC6; \
      int iNumMultALLOC7; \
      int iNumMultALLOC8; \
      int iNumMultALLOC9; \
      int iNumMultALLOC10; \
      int iNumRowALLOC; \
      iNumRowALLOC = iNumRow + 1; \
      iNumColALLOC = iNumCol + 1; \
      iNumDep1ALLOC = iNumDep1 + 1; \
      iNumDep2ALLOC = iNumDep2 + 1; \
      iNumDep3ALLOC = iNumDep3 + 1; \
      iNumMultALLOC1 = iNumRowALLOC * iNumColALLOC; \
      iNumMultALLOC2 = iNumRowALLOC * iNumColALLOC * iNumDep1ALLOC; \
      iNumMultALLOC3 = iNumRowALLOC * iNumColALLOC * iNumDep1ALLOC \
          * iNumDep2ALLOC; \
      iNumMultALLOC4 = iNumRowALLOC * iNumColALLOC * iNumDep1ALLOC \
          * iNumDep2ALLOC * iNumDep3ALLOC; \
      iNumMultALLOC5 = iNumColALLOC * iNumDep1ALLOC; \
      iNumMultALLOC6 = iNumColALLOC * iNumDep1ALLOC * iNumDep2ALLOC; \
      iNumMultALLOC7 = iNumColALLOC * iNumDep1ALLOC * iNumDep2ALLOC \
          * iNumDep3ALLOC; \
      iNumMultALLOC8 = iNumDep1ALLOC * iNumDep2ALLOC; \
      iNumMultALLOC9 = iNumDep1ALLOC * iNumDep2ALLOC * iNumDep3ALLOC; \
      iNumMultALLOC10 = iNumDep2ALLOC * iNumDep3ALLOC; \
      aVar = (TYPE*****) \
          malloc( (unsigned int) ( iNumRowALLOC + 1 ) * sizeof( TYPE**** ) ); \
      if ( aVar == NULL ) \
      { \
        fprintf( stderr, "ALLOC5: Memory problem...\n" ); \
      } \
      aVar += 1; \
      aVar[0] = (TYPE****) \
          malloc( (unsigned int) ( iNumMultALLOC1 + 1 ) * sizeof( TYPE*** ) ); \
      if ( aVar[0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC5: Memory problem...\n" ); \
      } \
      aVar[0] += 1; \
      aVar[0][0] = (TYPE***) \
          malloc( (unsigned int) ( iNumMultALLOC2 + 1 ) * sizeof( TYPE** ) ); \
      if ( aVar[0][0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC5: Memory problem...\n" ); \
      } \
      aVar[0][0] += 1; \
      aVar[0][0][0] = (TYPE**) \
          malloc( (unsigned int) ( iNumMultALLOC3 + 1 ) * sizeof( TYPE* ) ); \
      if ( aVar[0][0][0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC5: Memory problem...\n" ); \
      } \
      aVar[0][0][0] += 1; \
      aVar[0][0][0][0] = (TYPE*) \
          malloc( (unsigned int) ( iNumMultALLOC4 + 1 ) * sizeof( TYPE ) ); \
      if ( aVar[0][0][0][0] == NULL ) \
      { \
        fprintf( stderr, "ALLOC5: Memory problem...\n" ); \
      } \
      aVar[0][0][0][0] += 1; \
      for ( iCALLOC1 = 0; iCALLOC1 <= iNumRow; iCALLOC1++ ) \
      { \
        if ( iCALLOC1 != 0 ) \
        { \
          aVar[iCALLOC1] = aVar[iCALLOC1 - 1] + iNumColALLOC; \
          aVar[iCALLOC1][0] = aVar[iCALLOC1 - 1][0] + iNumMultALLOC5; \
          aVar[iCALLOC1][0][0] = aVar[iCALLOC1 - 1][0][0] + iNumMultALLOC6; \
          aVar[iCALLOC1][0][0][0] = \
              aVar[iCALLOC1 - 1][0][0][0] + iNumMultALLOC7; \
        } \
        for ( iCALLOC2 = 0; iCALLOC2 <= iNumCol; iCALLOC2++ ) \
        { \
          if ( iCALLOC2 != 0 ) \
          { \
            aVar[iCALLOC1][iCALLOC2] = aVar[iCALLOC1][iCALLOC2 - 1] \
                + iNumDep1ALLOC; \
            aVar[iCALLOC1][iCALLOC2][0] = aVar[iCALLOC1][iCALLOC2 - 1][0] \
                + iNumMultALLOC8; \
            aVar[iCALLOC1][iCALLOC2][0][0] = \
                aVar[iCALLOC1][iCALLOC2 - 1][0][0] + iNumMultALLOC9; \
          } \
          for ( iCALLOC3 = 0; iCALLOC3 <= iNumDep1; iCALLOC3++ ) \
          { \
            if ( iCALLOC3 != 0 ) \
            { \
              aVar[iCALLOC1][iCALLOC2][iCALLOC3] = \
                  aVar[iCALLOC1][iCALLOC2][iCALLOC3 - 1] + iNumDep2ALLOC; \
              aVar[iCALLOC1][iCALLOC2][iCALLOC3][0] = \
                  aVar[iCALLOC1][iCALLOC2][iCALLOC3 - 1][0] + iNumMultALLOC10; \
            } \
            for ( iCALLOC4 = 0; iCALLOC4 <= iNumDep2; iCALLOC4++ ) \
            { \
              if ( iCALLOC4 != 0 ) \
              { \
                aVar[iCALLOC1][iCALLOC2][iCALLOC3][iCALLOC4] = \
                    aVar[iCALLOC1][iCALLOC2][iCALLOC3][iCALLOC4 - 1] + iNumDep3ALLOC; \
              } \
            } \
          } \
        } \
      } \
    } \
    /* Allocate the memory for a 5D TYPE matrix between (0,0,0,0,0) and \
        (iNumRow,iNumCol,iNumDep1,iNumDep2,iNumDep3) */

#define DEALLOC1( pVar ) \
    { \
      free( (char*) ( pVar ) ); \
      pVar = NULL; \
    } \
    /* Deallocate the memory for the 1D matrix */

#define DEALLOC2( aVar ) \
    { \
      free( (char*) ( aVar[0] - 1 ) ); \
      aVar[0] = NULL; \
      free( (char*) ( aVar - 1 ) ); \
      aVar = NULL; \
    } \
    /* Deallocate the memory for the 2D matrix */

#define DEALLOC3( aVar ) \
    { \
      free( (char*) ( aVar[0][0] - 1 ) ); \
      aVar[0][0] = NULL; \
      free( (char*) ( aVar[0] - 1 ) ); \
      aVar[0] = NULL; \
      free( (char*) ( aVar - 1 ) ) ; \
      aVar = NULL; \
    } \
    /* Deallocate the memory for the 3D matrix */

#define DEALLOC4( aVar ) \
    { \
      free( (char*) ( aVar[0][0][0] - 1 ) ); \
      aVar[0][0][0] = NULL; \
      free( (char*) ( aVar[0][0] - 1 ) ); \
      aVar[0][0] = NULL; \
      free( (char*) ( aVar[0] - 1 ) ); \
      aVar[0] = NULL; \
      free( (char*) ( aVar - 1 ) ) ; \
      aVar = NULL; \
    } \
    /* Deallocate the memory for the 4D matrix */

#define DEALLOC5( aVar ) \
    { \
      free( (char*) ( aVar[0][0][0][0] - 1 ) ); \
      aVar[0][0][0][0] = NULL; \
      free( (char*) ( aVar[0][0][0] - 1 ) ); \
      aVar[0][0][0] = NULL; \
      free( (char*) ( aVar[0][0] - 1 ) ); \
      aVar[0][0] = NULL; \
      free( (char*) ( aVar[0] - 1 ) ); \
      aVar[0] = NULL; \
      free( (char*) ( aVar - 1 ) ) ; \
      aVar = NULL; \
    } \
    /* Deallocate the memory for the 5D matrix */

/* -------------------------------------------------------------------------- */

/* Function declarations */

int ALLOC1_double( double **adfVar, int iNumRow );
int ALLOC2_double( double ***adfVar, int iNumRow, int iNumCol );
int ALLOC3_double( double ****adfVar, int iNumRow, int iNumCol, int iNumDep );
int ALLOC4_double( double *****adfVar, int iNumRow, int iNumCol, int iNumDep1,
    int iNumDep2 );
int ALLOC5_double( double ******adfVar, int iNumRow, int iNumCol, int iNumDep1,
    int iNumDep2, int iNumDep3 );
int DEALLOC1_double( double **adfVar );
int DEALLOC2_double( double ***adfVar );
int DEALLOC3_double( double ****adfVar );
int DEALLOC4_double( double *****adfVar );
int DEALLOC5_double( double ******adfVar );
