
/* ------------------------------------------------------------------------- */

/*

message.c

United States Naval Observatory
USNO/NRL Optical Interferometer
3450 Massachusetts Avenue, NW
Washington, DC  20392-5420

Subsystem:
----------
My software libraries

Description:
------------
This file contains the error recovery/halt functions.

Functions:
----------
NrError, NrWarning.

Modification history:
---------------------
1992 Jul 27 - Nicholas Elias, USNO/AD5
              File pilfered from "Numerical Recipes in C" and modified.
1992 Dec 10 - Nicholas Elias, USNO/AD5
              Function NrWarning( ) added.

*/

/* ------------------------------------------------------------------------- */

/* Include files */

#include <stdio.h>    /* Standard I/O */
#include <stdlib.h>   /* Standard library */
#include <math.h>     /* Math */

#include "message.h"  /* Message function prototypes */

/* ------------------------------------------------------------------------- */

/*

NrError

Description:
------------
This function prints an error message supplied by the user and returns control
to the calling function.  It is suggested that the message be of the format
"!! FunctionName: {Error message}...".  The function returns a value of 1 to
the operating system.

Inputs:
-------
*psErrorText - This is the pointer to the string which contains the error
               message.

Outputs:
--------
None.

Modification history:
---------------------
1992 Apr 03 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* ------------------------------------------------------------------------- */

void NrError( char *psErrorText )

{

  /* Print the error message */

  fprintf( stdout, "%s \n", psErrorText );


  /* Terminate execution and return a value of 1 */

  exit( 1 );

}

/* ------------------------------------------------------------------------- */

/*

NrWarning

Description:
------------
This function prints a warning message supplied by the user and returns
control to the calling function.  It is suggested that the message be of
the format "!! FunctionName: {Warning message}...".

Inputs:
-------
*psWarningText - This is the pointer to the string which contains the warning
                 message.

Outputs:
--------
None.

Modification history:
---------------------
1992 Apr 03 - Nicholas Elias, USNO/AD5
              Function created.

*/

/* ------------------------------------------------------------------------- */

void NrWarning( char *psWarningText )

{

  /* Print the warning message */

  fprintf( stdout, "%s \n", psWarningText );

}

