
# ---------------------------------------------------------------------------- #

# rcol.tcl

# USNO/NRL Optical Interferometer
# United States Naval Observatory
# 3450 Massachusetts Avenue, NW
# Washington, DC  20392-5420

# Subsystem:
# ----------
# ptcl

# Description:
# ------------
# This file contains a tcl procedure that reads in data by columns.

# tcl procedures:
# ---------------
# rcol.

# Modification history:
# ---------------------
# 1995 Dec 07 - Nicholas Elias, USNO/AD5
#               File created with tcl procedure rcol{ }.

# ---------------------------------------------------------------------------- #

# rcol

# Description:
# ------------
# This tcl procedure reads simple ASCII files arranged in columns.

# Inputs:
# -------
# File    - This is the ASCII file name.
# ColList - This is the column list.  In the file, the leftmost column is 0, and
#           so on.

# Outputs:
# --------
# List - This is the list of column lists.  The first sublist is the data from
#        the first specified column, etc.  If an error occurs, a zero is
#        returned.

# Modification history:
# ---------------------
# 1995 Dec 07 - Nicholas Elias, USNO/AD5
#               tcl procedure created.

# ---------------------------------------------------------------------------- #

proc rcol {File ColList} {

  # Open the file #

  if ![file exists $File] {
    puts "!! rcol: Input file does not exist..."
    return 0
  }

  if ![file readable $File] {
    puts "!! rcol: Input file is not readable..."
    return 0
  }

  set Stream [open $File r]


  # Initialize the sublists #

  foreach Col $ColList {
    set SubList($Col) ""
  }


  # Get the data from the file #

  set NumLine [lindex [exec wc -l $File] 0]

  for {set Line 1} {$Line <= $NumLine} {incr Line} {

    gets $Stream LineText

    if {[string range $LineText 0 0] == "#"} {
      continue
    }

    foreach Col $ColList {
      lappend SubList($Col) [lindex $LineText $Col]
    }

  }


  # Close the file #

  close $Stream


  # Create the final list of sublists #

  set List ""

  foreach Col $ColList {
    lappend List $SubList($Col)
  }


  # Return the list of sublists #

  return $List

}

